"""Console script for acdh_collatex_utils."""
import click

from . acdh_collatex_utils import CxCollate


@click.command()  # pragma: no cover
@click.option('-g', '--glob-pattern', default='./fixtures/*.xml', show_default=True)  # pragma: no cover
@click.option('--nr/--r', default=False, show_default=True)  # pragma: no cover
def collate(glob_pattern, nr):  # pragma: no cover
    """Console script to flatten XML/TEI files of a work."""
    new_glob_pattern = f"{glob_pattern}"
    print(new_glob_pattern)
    output_dir = new_glob_pattern.replace("*.xml", 'collated')

    out = CxCollate(
        glob_pattern=new_glob_pattern,
        glob_recursive=nr,
        output_dir=output_dir,
        char_limit=False
    ).collate()
    for x in out:
        click.echo(
            click.style(
                f"finished saving: {x}\n",
                fg='green'
            )
        )

    click.echo(
        click.style(
            "\n################################\n",
            fg='green'
        )
    )



#!/usr/bin/env python

"""Tests for `acdh_collatex_utils` package."""

import os
import unittest

from acdh_collatex_utils.chunks import get_chunks
from acdh_collatex_utils.acdh_collatex_utils import CHUNK_SIZE

file = os.path.join('.', 'fixtures', 'lorem_ipsum.txt')

with open(file) as f:
    lorem_ipsum = f.read()


class TestAcdh_collatex_utils(unittest.TestCase):
    """Tests for `acdh_collatex_utils` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_file_loading(self):
        self.assertTrue(type(lorem_ipsum), 'list')

    def test_0002_chunk_text(self):
        """ test chunk function """
        chunker = get_chunks(lorem_ipsum, CHUNK_SIZE, 'some_id.txt')
        chunks = [x for x in chunker]
        self.assertTrue(
            type(chunks),
            'list'
        )
        self.assertTrue(
            type(chunks[0]),
            'dict'
        )



import glob
import os

import lxml.etree as ET
import pandas as pd
import collatex
from collatex.core_functions import export_alignment_table_as_tei
from datetime import datetime

from acdh_xml_pyutils.xml import XMLReader
from . chunks import get_chunks
from . collatex_patch import visualize_table_vertically_with_colors

CUSTOM_XSL = os.path.join(
    os.path.dirname(__file__),
    "xslt",
    "only_text.xsl"
)

CHUNK_SIZE = 2000

XPATH = './/tei:body//tei:p//text()'


class CxReader(XMLReader):

    """ Class to parse and preprocess XML/TEI files for CollateX
    """

    def preprocess(self):

        """
        preprocessing like e.g. removing of obstrusive inline tags `<tei:lb break="no"/>`

        :return: A cleaned lxml.ElementTree
        """
        transform = ET.XSLT(self.custom_xsl)
        new = transform(self.tree)
        return new

    def plain_text(self):

        """ fetches plain text of processed XML/TEI doc

        :return: The processed XML/TEIs plain text
        :rtype: str

        """
        cur_doc = self.cur_doc
        plain_text = " ".join(cur_doc.xpath(self.xpath, namespaces=self.nsmap))
        plain_text = " ".join(plain_text.split())
        if self.char_limit:
            return plain_text[:5000]
        return plain_text

    def yield_chunks(self):

        """ yields chunks of the object's plain text

        :return:  yields dicts like\
            `{"id": "example_text.txt", "chunk_nr": "001", "text": "text of chunk", "char_count": 13}`
        :rtype: dict
        """
        return get_chunks(self.plain_text, self.chunk_size, self.file_name)

    def __init__(
        self,
        custom_xsl=CUSTOM_XSL,
        char_limit=True,
        chunk_size=CHUNK_SIZE,
        to_compare_xpath=XPATH,
        **kwargs
    ):

        """ initializes the class

        :param custom_xsl: Path to XSLT file used for\
        processing TEIs to return needed (plain) text
        :type custom_xsl: str

        :param char_limit: Should the number of chars of the\
        plaintext be limited?
        :type custom_xsl: bool

        :param chunk_size: Size of chunks yield by `yield_chunks`
        :type custom_xsl: int

        :return: A CxReader instance
        :rtype: `acdh_collatex_utils.CxReader`

        """
        super().__init__(**kwargs)
        self.xpath = to_compare_xpath
        self.custom_xsl = ET.parse(custom_xsl)
        self.char_limit = char_limit
        self.chunk_size = chunk_size
        self.cur_doc = self.preprocess()
        self.plain_text = self.plain_text()
        self.collatex_wit = (self.file, self.plain_text)
        self.plaint_text_len = len(self.plain_text)
        try:
            self.file_name = os.path.split(self.file)[1]
        except Exception as e:
            print(f"spliitng `self.file_name` did not work due to {e}")
            self.file_name = self.file


def yield_chunks(files, chunk_size=CHUNK_SIZE, char_limit=True, xpath=XPATH):

    """ utility function to yield chunks from a collection of files

    :param files: List of full file names / file paths to TEI/XML
    :type files: list

    :return: yields chunk dicts
    :rtype: dict

    """
    for x in files:
        doc = CxReader(xml=x, char_limit=char_limit, chunk_size=chunk_size, to_compare_xpath=xpath)
        chunks = doc.yield_chunks()
        for y in chunks:
            yield y


def chunks_to_df(files, char_limit=True, chunk_size=CHUNK_SIZE, xpath=XPATH):

    """ reads chunks from a list of files into a `pandas.DataFrame`

    :param files: List of full file names / file paths to TEI/XML
    :type files: list

    :return: a pandas.Dataframe with columns `id`, `chunk_nr`, `text`, and `char_count`
    :rtype: pandas.Dataframe
    """
    df = pd.DataFrame(yield_chunks(files, char_limit=char_limit, chunk_size=chunk_size, xpath=xpath))
    return df


class CxCollate():
    """ Class collate a collection of XML/TEI files and store the results
    """

    def chunk_df(self):
        return chunks_to_df(self.files, char_limit=self.char_limit, chunk_size=self.chunk_size, xpath=self.xpath)

    def collate(self):
        print(f"start collating {self.file_count} Documents at {datetime.now()}\n")
        print("################################\n")
        files = []
        try:
            os.makedirs(self.output_dir)
        except Exception as e:  # noqa: F841
            print(f'{self.output_dir}: {self.output_dir} already exists')
        counter = 0
        df = self.df
        for gr in df.groupby('chunk_nr'):
            counter += 1
            start_time = datetime.now()
            print(f"start collating group {counter} at {start_time}")
            f_html = os.path.join(self.output_dir, f"out__{counter:03}.html")
            f_tei = os.path.join(self.output_dir, f"out__{counter:03}.tei")
            collation = collatex.Collation()
            cur_df = gr[1]
            for i, row in cur_df.iterrows():
                print(row['id'])
                collation.add_plain_witness(row['id'], row['text'])
            table = collatex.collate(collation)
            end_time = datetime.now()
            print(f"finished at {end_time} after {end_time - start_time}")
            print(f"saving results to {f_html} and {f_tei}")
            with open(f_html, 'w') as f:
                print(
                    visualize_table_vertically_with_colors(
                        table,
                        collation
                    ),
                    file=f
                )
            with open(f_tei, 'w') as f:
                print(
                    export_alignment_table_as_tei(
                        table,
                        indent=True
                    ),
                    file=f
                )
            files.append([f_html, f_tei])
        print("################################\n")
        print(f"finished collating {self.file_count} Documents at {datetime.now()}\n")
        print("################################\n")
        return files

    def __init__(
        self,
        glob_pattern="./fixtures/*.xml",
        glob_recursive=False,
        output_dir="./tmp",
        custom_xsl=CUSTOM_XSL,
        char_limit=True,
        chunk_size=CHUNK_SIZE,
        to_compare_xpath=XPATH,
        **kwargs
    ):

        """ initializes the class

        :param glob_pattern: a `glob.glob` pattern to retrieve a list of files
        :type glob_pattern: 'str'

        :param glob_recursive: should the glob pattern be recursive
        :type glob_recursive: bool

        :param output_dir: Location to store collation results
        :type output_dir: str

        :param custom_xsl: Path to XSLT file used for\
        processing TEIs to return needed (plain) text
        :type custom_xsl: str

        :param char_limit: Should the number of chars of the\
        plaintext be limited?
        :type custom_xsl: bool

        :param chunk_size: Size of chunks yield by `yield_chunks`
        :type custom_xsl: int

        :return: A CxReader instance
        :rtype: `acdh_collatex_utils.CxReader`
        """
        self.xpath = to_compare_xpath
        self.glob_pattern = glob_pattern
        self.glob_recursive = glob_recursive
        self.output_dir = output_dir
        self.custom_xsl = ET.parse(custom_xsl)
        self.char_limit = char_limit
        self.chunk_size = chunk_size
        self.files = glob.glob(self.glob_pattern, recursive=False)
        self.file_count = len(self.files)
        self.df = self.chunk_df()



from collatex.display_module import Table, TableCell, fill, TableRow


def visualize_table_vertically_with_colors(table, collation):
    """ patch of `collatex.display_module.visualize_table_vertically_with_colors` function
    """
    # print the table vertically
    # switch columns and rows
    rows = []
    for column in table.columns:
        cells = []
        for witness in collation.witnesses:
            cell = column.tokens_per_witness.get(witness.sigil)
            cells.append(
                TableCell(
                    text=fill("".join(item.token_data["t"] for item in cell) if cell else "-", 20),
                    bgcolor="white" if column.variant else "00FFFF"
                )
            )
        rows.append(TableRow(cells=cells))
    sigli = []
    for witness in collation.witnesses:
        sigli.append(witness.sigil)
    x = Table(header_row=sigli, rows=rows)
    return str(x)



import os
import lxml.etree as ET
from acdh_tei_pyutils.tei import TeiReader
from bs4 import BeautifulSoup, Tag


LEM_XSL = os.path.join(
    os.path.dirname(__file__),
    "xslt",
    "lem.xsl"
)


TEI_DUMMY_STRING = """
<?xml version="1.0" encoding="UTF-8"?>
<TEI xmlns="http://www.tei-c.org/ns/1.0">
  <teiHeader>
      <fileDesc>
         <titleStmt>
            <title>Title</title>
         </titleStmt>
         <publicationStmt>
            <p>Publication Information</p>
         </publicationStmt>
         <sourceDesc>
            <listWit/>
         </sourceDesc>
      </fileDesc>
  </teiHeader>
  <text>
      <body>
         <ab/>
      </body>
  </text>
</TEI>
"""


def merge_html_fragments(files):
    """ merges splitted collation tables into single (X)HTML file

    :param files: A list of absolute file paths

    :return: a beautiful soup object providing an html-table
    """

    tr = []
    for x in sorted(files):
        with open(x, 'r') as f:
            soup = BeautifulSoup(f, "html.parser")
            for row in soup.find_all('tr'):
                tr.append(row)
    new_soup = BeautifulSoup()
    table = Tag(new_soup, name="table")
    new_soup.append(table)
    for x in tr:
        table.append(x)
    return new_soup


def merge_tei_fragments(files):
    """ takes a list of files (fullpaths) and retuns a single tei:ab element.etree node"""
    full_doc = ET.Element("{http://www.tei-c.org/ns/1.0}ab", nsmap={None: "http://www.tei-c.org/ns/1.0"})
    for x in sorted(files):
        doc = TeiReader(x)
        for rdg in doc.any_xpath('.//tei:rdg'):
            old_ids = rdg.attrib['wit'].split()
            new_ids = " ".join([f"#{x[7:]}" for x in old_ids])
            rdg.attrib['wit'] = new_ids
        for node in doc.any_xpath('./*'):
            full_doc.append(node)
    return full_doc


def make_full_tei_doc(input_file, wit_prefix=None):
    """ takes the rusult of merged collated tei fragments\
        and returns a valid TEI document as TeiReader object"""
    tei_dummy = TeiReader(TEI_DUMMY_STRING)
    crit_app = TeiReader(input_file)
    body = tei_dummy.any_xpath('.//tei:ab')[0]
    list_wit_node = tei_dummy.any_xpath('.//tei:listWit')[0]
    wit_set = set()

    for rdg in crit_app.any_xpath('.//tei:rdg/@wit'):
        for w in rdg.split():
            wit_set.add(w[1:])

    for x in list(sorted(wit_set)):
        if wit_prefix is not None:
            wit_id = f"{wit_prefix}__{x}"
        else:
            wit_id = x
        w_node = ET.Element("{http://www.tei-c.org/ns/1.0}witness", nsmap={None: "http://www.tei-c.org/ns/1.0"})
        w_node.attrib['{http://www.w3.org/XML/1998/namespace}id'] = wit_id
        w_node.text = wit_id
        list_wit_node.append(w_node)
    for x in crit_app.any_xpath('./*'):
        body.append(x)
    return tei_dummy


def make_positive_app(input_file):
    """ transform negativ apparat into positive one
    :param input_file: A TEI/XML file with listWit and app elements

    :return: A TeiReader instance
    """
    doc = TeiReader(input_file)
    wit_ids = set(doc.any_xpath('.//tei:witness/@xml:id'))
    for x in doc.any_xpath('.//tei:app'):
        cur_wit_ids = set()
        for wit in x.xpath('./*'):
            for w in wit.attrib['wit'].split(' '):
                cur_wit_ids.add(w[1:])
        missing = list(wit_ids.difference(cur_wit_ids))
        if missing:
            rdg = ET.Element('{http://www.tei-c.org/ns/1.0}rdg')
            rdg.attrib['wit'] = " ".join([f"#{x}" for x in missing])
            x.append(rdg)
    return doc


def define_readings(input_file, rdg_wit_id, lem_xsl=LEM_XSL):
    """ transforms tei:rdg[@weit='rdg_wit_id'] into tei:lem

    :param input_file: XML/TEI file with tei:app
    :param rdg_wit_id: the xml:id of the witness for the lemma
    :param lem_xsl: The XSLT doing the transformation

    :return: the output of the transformation
    """
    with open(lem_xsl, 'r', encoding='utf-8') as f:
        proper_xsl = f.read().replace('!!!REPLACEME!!!', rdg_wit_id)
    input_doc = TeiReader(input_file)
    xsl_doc = TeiReader(proper_xsl)
    transform = ET.XSLT(xsl_doc.tree)
    out = transform(input_doc.tree)
    return out



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()


requirements = [
    'acdh-tei-pyutils>=0.25.0,<1',
    'beautifulsoup4>=4.11.1,<5',
    'click>=7.1<9',
    'collatex==2.2',
    'pandas>=1.1.5,<2',
    'python-Levenshtein',
    'tqdm>=4.52.0,<5'
]

setup_requirements = []

test_requirements = []

setup(
    author="Peter Andorfer",
    author_email='peter.andorfer@oeaw.ac.at',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.8',
    ],
    description="Utility functions to work with collatex",
    entry_points={
        'console_scripts': [
            'collate=acdh_collatex_utils.cli:collate',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    package_data={
        '': ['fixtures/*.*'],
        'acdh_collatex_utils': ['xslt/*.xsl']
    },
    keywords='acdh_collatex_utils',
    name='acdh_collatex_utils',
    packages=find_packages(include=['acdh_collatex_utils', 'acdh_collatex_utils.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/acdh-oeaw/acdh_collatex_utils',
    version="v1.12",
    zip_safe=False,
)
